@@ -1,49 +1,21 @@
 /*
+ * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @license AGPL-3.0
  *
- *    OPEN-XCHANGE legal information
+ * This code is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- *    All intellectual property rights in the Software are protected by
- *    international copyright laws.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Affero General Public License for more details.
  *
+ * You should have received a copy of the GNU Affero General Public License
+ * along with OX App Suite.  If not, see <https://www.gnu.org/licenses/agpl-3.0.txt>.
  *
- *    In some countries OX, OX Open-Xchange, open xchange and OXtender
- *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the OX Software GmbH group of companies.
- *    The use of the Logos is not covered by the GNU General Public License.
- *    Instead, you are allowed to use these Logos according to the terms and
- *    conditions of the Creative Commons License, Version 2.5, Attribution,
- *    Non-commercial, ShareAlike, and the interpretation of the term
- *    Non-commercial applicable to the aforementioned license is published
- *    on the web site http://www.open-xchange.com/EN/legal/index.html.
- *
- *    Please make sure that third-party modules and libraries are used
- *    according to their respective licenses.
- *
- *    Any modifications to this package must retain all copyright notices
- *    of the original copyright holder(s) for the original code used.
- *
- *    After any such modifications, the original and derivative code shall remain
- *    under the copyright of the copyright holder(s) and/or original author(s)per
- *    the Attribution and Assignment Agreement that can be located at
- *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
- *    given Attribution for the derivative code and a license granting use.
- *
- *     Copyright (C) 2016-2020 OX Software GmbH
- *     Mail: info@open-xchange.com
- *
- *
- *     This program is free software; you can redistribute it and/or modify it
- *     under the terms of the GNU General Public License, Version 2 as published
- *     by the Free Software Foundation.
- *
- *     This program is distributed in the hope that it will be useful, but
- *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
- *     for more details.
- *
- *     You should have received a copy of the GNU General Public License along
- *     with this program; if not, write to the Free Software Foundation, Inc., 59
- *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Any use of the work other than as authorized under this license or copyright law is prohibited.
  *
  */
 
@@ -53,7 +25,6 @@ import java.io.IOException;
 import java.io.PrintWriter;
 import java.util.HashMap;
 import java.util.Map;
-import java.util.UUID;
 import javax.servlet.http.Cookie;
 import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;
@@ -71,10 +42,8 @@ import com.openexchange.authentication.LoginExceptionCodes;
 import com.openexchange.context.ContextService;
 import com.openexchange.exception.OXException;
 import com.openexchange.groupware.contexts.Context;
-import com.openexchange.groupware.notify.hostname.HostnameService;
 import com.openexchange.java.Charsets;
 import com.openexchange.java.Strings;
-import com.openexchange.java.util.UUIDs;
 import com.openexchange.login.LoginRequest;
 import com.openexchange.login.LoginResult;
 import com.openexchange.login.internal.LoginMethodClosure;
@@ -93,8 +62,6 @@ import com.openexchange.session.reservation.EnhancedAuthenticated;
 import com.openexchange.session.reservation.Reservation;
 import com.openexchange.session.reservation.SessionReservationService;
 import com.openexchange.tools.servlet.AjaxExceptionCodes;
-import com.openexchange.tools.servlet.http.Cookies;
-import com.openexchange.tools.servlet.http.Tools;
 import com.openexchange.tools.session.ServerSessionAdapter;
 import com.openexchange.user.User;
 import com.openexchange.user.UserService;
@@ -224,17 +191,12 @@ public class OIDCLoginRequestHandler implements LoginRequestHandler {
          * @param request The request
          * @param session The session to get the redirect location for
          * @param reservation The reservation
-         * @param requestContext The login request context
          * @return The relative redirect location of the web frontend.
          * @throws OXException
-         * @throws IOException
          */
-        private String getRedirectLocationForSession(HttpServletRequest request, Session session, Reservation reservation, LoginRequestContext requestContext) throws OXException, IOException {
+        private String getRedirectLocationForSession(HttpServletRequest request, Session session, Reservation reservation) throws OXException {
             LOG.trace("getRedirectLocationForSession(HttpServletRequest request: {}, Session session: {}, Reservation reservation: {})", request.getRequestURI(), session.getSessionID(), reservation.getToken());
             OIDCTools.validateSession(session, request);
-            if (session.getContextId() != reservation.getContextId() && session.getUserId() != reservation.getUserId()) {
-                this.handleException(null, true, LoginExceptionCodes.LOGIN_DENIED.create(), 0, requestContext);
-            }
             return OIDCTools.buildFrontendRedirectLocation(session, OIDCTools.getUIWebPath(backend.getLoginConfiguration(), backend.getBackendConfig()), request.getParameter(OIDCTools.PARAM_DEEP_LINK));
         }
 
@@ -244,35 +206,43 @@ public class OIDCLoginRequestHandler implements LoginRequestHandler {
          * @param request The request
          * @param response The response
          * @param reservation The reservation
-         * @param oidcAtologinCookie The autologin cookie
+         * @param sessionCookie The session cookie
          * @param respondWithJson True to response with JSON data, false to send a redirect
          * @param requestContext The login request context
          * @return true on success, false otherwise
          * @throws IOException
          */
-        private boolean getAutologinByCookieURL(HttpServletRequest request, HttpServletResponse response, Reservation reservation, Cookie oidcAtologinCookie, boolean respondWithJson, LoginRequestContext requestContext) throws IOException {
-            LOG.trace("getAutologinByCookieURL(HttpServletRequest request: {}, HttpServletResponse response, Reservation reservation.token: {}, Cookie oidcAtologinCookie: {}, boolean respondWithJson)", request.getRequestURI(), reservation.getToken(), oidcAtologinCookie != null ? oidcAtologinCookie.getValue() : "null", respondWithJson ? Boolean.TRUE : Boolean.FALSE);
-            if (oidcAtologinCookie != null) {
+        private boolean getAutologinByCookieURL(HttpServletRequest request, HttpServletResponse response, Reservation reservation, Cookie sessionCookie, boolean respondWithJson, LoginRequestContext requestContext) throws IOException {
+            LOG.trace("getAutologinByCookieURL(HttpServletRequest request: {}, HttpServletResponse response, Reservation reservation.token: {}, Cookie open-xchange-session Cookie: {}, boolean respondWithJson)", request.getRequestURI(), reservation.getToken(), sessionCookie != null ? sessionCookie.getValue() : "null", respondWithJson ? Boolean.TRUE : Boolean.FALSE);
+            if (sessionCookie != null) {
                 try {
-                    Session session = OIDCTools.getSessionFromAutologinCookie(oidcAtologinCookie, request);
+                    Session session = OIDCTools.getSessionFromSessionCookie(sessionCookie, request);
                     if (session != null) {
+                        int expectedContextId = reservation.getContextId();
+                        int expectedUserId = reservation.getUserId();
+                        if (expectedContextId != session.getContextId() || expectedUserId != session.getUserId()) {
+                            // wrong session
+                            LOG.debug("Session {} does not match expected session for reservation {}.", session.getSessionID(), reservation.getToken());
+                            handleException(null, true, LoginExceptionCodes.LOGIN_DENIED.create(), 0, requestContext);
+                            return false;
+                        }
                         backend.updateSession(session, reservation.getState());
                         // the getRedirectLocationForSession does also the validation check of the session
                         if (respondWithJson) {
                             this.writeSessionDataAsJson(session, response);
                         } else {
-                            String redirectLocationForSession = getRedirectLocationForSession(request, session, reservation, requestContext);
+                            String redirectLocationForSession = getRedirectLocationForSession(request, session, reservation);
                             response.sendRedirect(redirectLocationForSession);
                         }
                         return true;
                     }
                     //No session found, log that
-                    LOG.debug("No session found for OIDC Cookie with value: {}", oidcAtologinCookie.getValue());
+                    LOG.debug("No session found for OIDC Cookie with value: {}", sessionCookie.getValue());
                 } catch (OXException | JSONException e) {
                     LOG.debug("Ignoring OIDC auto-login attempt due to failed IP or secret check", e);
                 }
 
-                Cookie toRemove = (Cookie) oidcAtologinCookie.clone();
+                Cookie toRemove = (Cookie) sessionCookie.clone();
                 toRemove.setMaxAge(0);
                 response.addCookie(toRemove);
             }
@@ -281,8 +251,8 @@ public class OIDCLoginRequestHandler implements LoginRequestHandler {
 
         /**
          * Perform an Appsuite login. If autologin is enabled, the user will be logged in into his session. The
-         * autologin mechanism will check if a valid OIDCCookie is available with all needed information. The
-         * method will redirect the user to the Appsuite UI afterwards. If <code>respondWithJson</code> is set true,
+         * autologin mechanism will check if a valid open-xchange-session cookie is available with all needed information.
+         * The method will redirect the user to the Appsuite UI afterwards. If <code>respondWithJson</code> is set true,
          * the redirect location will be wrapped into a JSON Object.
          *
          * @param request The {@link HttpServletRequest}
@@ -293,17 +263,23 @@ public class OIDCLoginRequestHandler implements LoginRequestHandler {
          * @throws OXException
          * @throws JSONException
          */
+        @SuppressWarnings("deprecation")
         private boolean performCookieLogin(HttpServletRequest request, HttpServletResponse response, Reservation reservation, boolean respondWithJson, LoginRequestContext requestContext) throws IOException {
             LOG.trace("performCookieLogin(HttpServletRequest request: {}, HttpServletResponse response, Reservation reservation.token: {}, boolean respondWithJson)", request.getRequestURI(), reservation.getToken(), respondWithJson ? Boolean.TRUE : Boolean.FALSE);
             AutologinMode autologinMode = OIDCBackendConfig.AutologinMode.get(backend.getBackendConfig().autologinCookieMode());
 
-            if (autologinMode == OIDCBackendConfig.AutologinMode.SSO_REDIRECT) {
-                try {
-                    Cookie autologinCookie = OIDCTools.loadAutologinCookie(request, backend.getLoginConfiguration());
+            if (autologinMode.equals(OIDCBackendConfig.AutologinMode.SSO_REDIRECT)) {
+                LOG.debug("Deprecated OIDC auto-login with a cookie called by session token {}.", reservation.getToken());
+                response.setHeader("Deprecation", "version=\"v7.10.5\"");
+            }
+
+            try {
+                Cookie autologinCookie = OIDCTools.loadSessionCookie(request, backend.getLoginConfiguration());
+                if (null != autologinCookie) {
                     return getAutologinByCookieURL(request, response, reservation, autologinCookie, respondWithJson, requestContext);
-                } catch (OXException e) {
-                    LOG.debug("Ignoring OIDC auto-login attempt due to failed IP or secret check", e);
                 }
+            } catch (OXException e) {
+                LOG.debug("Ignoring OIDC auto-login attempt due to failed IP or secret check", e);
             }
             return false;
         }
@@ -358,24 +334,14 @@ public class OIDCLoginRequestHandler implements LoginRequestHandler {
                 return;
             }
 
-            boolean autologinEnabled = backend.getBackendConfig().isAutologinEnabled();
-
-            String autologinCookieValue = null;
-            if (autologinEnabled) {
-                LOG.trace("Try OIDC auto-login with a cookie");
-                if (performCookieLogin(request, response, reservation, respondWithJson, requestContext)) {
-                    return;
-                }
-                autologinCookieValue = UUIDs.getUnformattedString(UUID.randomUUID());
+            LOG.trace("Try OIDC auto-login with a cookie");
+            if (performCookieLogin(request, response, reservation, respondWithJson, requestContext)) {
+                return;
             }
 
-            LoginResult result = loginUser(request, context, user, reservation.getState(), autologinCookieValue);
+            LoginResult result = loginUser(request, context, user, reservation.getState());
             Session session = performSessionAdditions(result, request, response, idToken);
 
-            if (autologinEnabled) {
-                response.addCookie(createOIDCAutologinCookie(request, session, autologinCookieValue));
-            }
-
             sendRedirect(session, request, response, respondWithJson);
         }
 
@@ -399,6 +365,8 @@ public class OIDCLoginRequestHandler implements LoginRequestHandler {
 
             LoginServlet.writeSecretCookie(request, response, session, session.getHash(), request.isSecure(), request.getServerName(), backend.getLoginConfiguration());
 
+            LoginServlet.writeSessionCookie(response, session, session.getHash(), request.isSecure(), request.getServerName());
+
             return session;
         }
 
@@ -424,31 +392,6 @@ public class OIDCLoginRequestHandler implements LoginRequestHandler {
             writer.flush();
         }
 
-        /**
-         * Creates an OIDC autlogin cookie from the given request object
-         *
-         * @param request The request to create the cookie
-         * @param session The session
-         * @param uuid A random uuid
-         * @return The cookie created from the given request
-         * @throws OXException
-         */
-        private Cookie createOIDCAutologinCookie(HttpServletRequest request, Session session, String uuid) throws OXException {
-            LOG.trace("createOIDCAutologinCookie(HttpServletRequest request: {}, Session session: {}, String uuid: {})", request.getRequestURI(), session.getSessionID(), uuid);
-            String hash = OIDCTools.calculateHash(request, backend.getLoginConfiguration());
-            Cookie oidcAutologinCookie = new Cookie(OIDCTools.AUTOLOGIN_COOKIE_PREFIX + hash, uuid);
-            oidcAutologinCookie.setPath("/");
-            oidcAutologinCookie.setSecure(Tools.considerSecure(request));
-            oidcAutologinCookie.setMaxAge(-1);
-
-            String domain = OIDCTools.getDomainName(request, Services.getOptionalService(HostnameService.class));
-            String cookieDomain = Cookies.getDomainValue(domain);
-            if (cookieDomain != null) {
-                oidcAutologinCookie.setDomain(cookieDomain);
-            }
-            return oidcAutologinCookie;
-        }
-
         /**
          * Sets a redirect to the given response object, either as HTTP redirect or JSON response
          *
@@ -478,12 +421,11 @@ public class OIDCLoginRequestHandler implements LoginRequestHandler {
          * @param context The context
          * @param user The user
          * @param state The state
-         * @param oidcAutologinCookieValue The value of the auto login cookie
          * @return The login result
          * @throws OXException
          */
-        private LoginResult loginUser(HttpServletRequest request, final Context context, final User user, final Map<String, String> state, final String oidcAutologinCookieValue) throws OXException {
-            LOG.trace("loginUser(HttpServletRequest request: {}, final Context context: {}, final User user: {}, final Map<String, String> state.size: {}, final String oidcAutologinCookieValue: {})", request.getRequestURI(), I(context.getContextId()), I(user.getId()), I(state.size()), oidcAutologinCookieValue);
+        private LoginResult loginUser(HttpServletRequest request, final Context context, final User user, final Map<String, String> state) throws OXException {
+            LOG.trace("loginUser(HttpServletRequest request: {}, final Context context: {}, final User user: {}, final Map<String, String> state.size: {})", request.getRequestURI(), I(context.getContextId()), I(user.getId()), I(state.size()));
             final LoginRequest loginRequest = backend.getLoginRequest(request, user.getId(), context.getContextId(), backend.getLoginConfiguration());
 
             final OIDCBackend colosureBackend = backend;
@@ -500,9 +442,6 @@ public class OIDCLoginRequestHandler implements LoginRequestHandler {
                             LOG.trace("doEnhanceSession(Session session: {})", ses.getSessionID());
                             SessionDescription session = (SessionDescription) ses;
                             session.setStaySignedIn(false);
-                            if (oidcAutologinCookieValue != null) {
-                                session.setParameter(OIDCTools.SESSION_COOKIE, oidcAutologinCookieValue);
-                            }
 
                             Map<String, String> params = new HashMap<>(state);
                             params.put(OIDCTools.BACKEND_PATH, colosureBackend.getPath());
