@@ -1,49 +1,21 @@
 /*
+ * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @license AGPL-3.0
  *
- *    OPEN-XCHANGE legal information
+ * This code is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- *    All intellectual property rights in the Software are protected by
- *    international copyright laws.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Affero General Public License for more details.
  *
+ * You should have received a copy of the GNU Affero General Public License
+ * along with OX App Suite.  If not, see <https://www.gnu.org/licenses/agpl-3.0.txt>.
  *
- *    In some countries OX, OX Open-Xchange, open xchange and OXtender
- *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the OX Software GmbH group of companies.
- *    The use of the Logos is not covered by the GNU General Public License.
- *    Instead, you are allowed to use these Logos according to the terms and
- *    conditions of the Creative Commons License, Version 2.5, Attribution,
- *    Non-commercial, ShareAlike, and the interpretation of the term
- *    Non-commercial applicable to the aforementioned license is published
- *    on the web site http://www.open-xchange.com/EN/legal/index.html.
- *
- *    Please make sure that third-party modules and libraries are used
- *    according to their respective licenses.
- *
- *    Any modifications to this package must retain all copyright notices
- *    of the original copyright holder(s) for the original code used.
- *
- *    After any such modifications, the original and derivative code shall remain
- *    under the copyright of the copyright holder(s) and/or original author(s)per
- *    the Attribution and Assignment Agreement that can be located at
- *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
- *    given Attribution for the derivative code and a license granting use.
- *
- *     Copyright (C) 2016-2020 OX Software GmbH
- *     Mail: info@open-xchange.com
- *
- *
- *     This program is free software; you can redistribute it and/or modify it
- *     under the terms of the GNU General Public License, Version 2 as published
- *     by the Free Software Foundation.
- *
- *     This program is distributed in the hope that it will be useful, but
- *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
- *     for more details.
- *
- *     You should have received a copy of the GNU General Public License along
- *     with this program; if not, write to the Free Software Foundation, Inc., 59
- *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Any use of the work other than as authorized under this license or copyright law is prohibited.
  *
  */
 package com.openexchange.oidc.impl;
@@ -111,7 +83,11 @@ import com.openexchange.oidc.tools.OIDCTools;
 import com.openexchange.rest.client.httpclient.HttpClientService;
 import com.openexchange.server.ServiceLookup;
 import com.openexchange.session.Session;
+import com.openexchange.session.oauth.RefreshResult;
+import com.openexchange.session.oauth.SessionOAuthTokenService;
+import com.openexchange.session.oauth.TokenRefreshConfig;
 import com.openexchange.session.reservation.SessionReservationService;
+import com.openexchange.sessiond.ExpirationReason;
 import com.openexchange.sessiond.SessionExceptionCodes;
 import com.openexchange.sessiond.SessiondService;
 import com.openexchange.tools.servlet.http.Tools;
@@ -182,12 +158,12 @@ public class OIDCWebSSOProviderImpl implements OIDCWebSSOProvider {
         String redirectURL = "";
         LOG.trace("getAutologinURLFromOIDCCookie(HttpServletRequest request: {}, HttpServletResponse response)", request.getRequestURI());
         try {
-            Cookie autologinCookie = OIDCTools.loadAutologinCookie(request, this.loginConfiguration);
+            Cookie sessionCookie = OIDCTools.loadSessionCookie(request, this.loginConfiguration);
 
-            if (autologinCookie == null) {
+            if (sessionCookie == null) {
                 return null;
             }
-            redirectURL = this.getAutologinByCookieURL(request, response, autologinCookie);
+            redirectURL = this.getAutologinByCookieURL(request, response, sessionCookie);
         } catch (OXException e) {
             LOG.debug("Failed to load autologin url for request: {}", request.getRequestURI());
         }
@@ -195,19 +171,43 @@ public class OIDCWebSSOProviderImpl implements OIDCWebSSOProvider {
         return redirectURL;
     }
 
-    private String getAutologinByCookieURL(HttpServletRequest request, HttpServletResponse response, Cookie oidcAtologinCookie) throws OXException {
-        LOG.trace("getAutologinByCookieURL(HttpServletRequest request: {}, HttpServletResponse response, Cookie oidcAtologinCookie: {})", request.getRequestURI(), oidcAtologinCookie != null ? oidcAtologinCookie.getValue() : "null");
-        if (oidcAtologinCookie != null) {
-            Session session = OIDCTools.getSessionFromAutologinCookie(oidcAtologinCookie, request);
+    private String getAutologinByCookieURL(HttpServletRequest request, HttpServletResponse response, Cookie sessionCookie) {
+        LOG.trace("getAutologinByCookieURL(HttpServletRequest request: {}, HttpServletResponse response, Cookie sessionCookie: {})", request.getRequestURI(), sessionCookie != null ? sessionCookie.getValue() : "null");
+        if (sessionCookie != null) {
+            Session session = OIDCTools.getSessionFromSessionCookie(sessionCookie, request);
             if (session != null) {
-                return this.getRedirectLocationForSession(request, session);
+                try {
+                    OIDCTools.validateSession(session, request);
+                    SessionOAuthTokenService sessionOAuthTokenService = services.getOptionalService(SessionOAuthTokenService.class);
+                    if (null != sessionOAuthTokenService) {
+                        OIDCBackendConfig config = backend.getBackendConfig();
+                        OIDCTokenRefresher refresher = new OIDCTokenRefresher(backend, session);
+                        TokenRefreshConfig refreshConfig = OIDCTools.getTokenRefreshConfig(config);
+                        try {
+                            RefreshResult result = sessionOAuthTokenService.checkOrRefreshTokens(session, refresher, refreshConfig);
+                            if (result.isSuccess()) {
+                                return this.getRedirectLocationForSession(request, session);
+                            }
+                            LOG.debug("Error checking and refreshing tokens for session {}", session.getSessionID());
+                            return null;
+                        } catch (InterruptedException e) {
+                            LOG.warn("Thread was interrupted while checking session oauth tokens", e);
+                            // keep interrupted state
+                            Thread.currentThread().interrupt();
+                        }
+                    }
+                    LOG.debug("Session {} found but SessionOAuthTokenService unavailable.", session.getSessionID());
+                } catch (OXException e) {
+                    LOG.debug(e.getMessage(), e);
+                }
+                return null;
             }
             //No session found, log that
-            LOG.debug("No valid session found for OIDC Cookie with value: {}", oidcAtologinCookie.getValue());
+            LOG.debug("No valid session found for OIDC Cookie with value: {}", sessionCookie.getValue());
         }
 
-        if (oidcAtologinCookie != null) {
-            Cookie toRemove = (Cookie) oidcAtologinCookie.clone();
+        if (sessionCookie != null) {
+            Cookie toRemove = (Cookie) sessionCookie.clone();
             toRemove.setMaxAge(0);
             response.addCookie(toRemove);
         }
@@ -469,7 +469,9 @@ public class OIDCWebSSOProviderImpl implements OIDCWebSSOProvider {
         //logout user
         Session session = this.getSessionFromId(sessionId);
         if (null == session) {
-            throw SessionExceptionCodes.SESSION_EXPIRED.create(sessionId);
+            OXException oxe = SessionExceptionCodes.SESSION_EXPIRED.create(sessionId);
+            oxe.setProperty(SessionExceptionCodes.OXEXCEPTION_PROPERTY_SESSION_EXPIRATION_REASON, ExpirationReason.NO_SUCH_SESSION.getIdentifier());
+            throw oxe;
         }
         return this.getRedirectForLogoutFromOXServer(session, request, response, logoutRequestInfo);
     }
