@@ -1,49 +1,21 @@
 /*
+ * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @license AGPL-3.0
  *
- *    OPEN-XCHANGE legal information
+ * This code is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- *    All intellectual property rights in the Software are protected by
- *    international copyright laws.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Affero General Public License for more details.
  *
+ * You should have received a copy of the GNU Affero General Public License
+ * along with OX App Suite.  If not, see <https://www.gnu.org/licenses/agpl-3.0.txt>.
  *
- *    In some countries OX, OX Open-Xchange, open xchange and OXtender
- *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the OX Software GmbH group of companies.
- *    The use of the Logos is not covered by the GNU General Public License.
- *    Instead, you are allowed to use these Logos according to the terms and
- *    conditions of the Creative Commons License, Version 2.5, Attribution,
- *    Non-commercial, ShareAlike, and the interpretation of the term
- *    Non-commercial applicable to the aforementioned license is published
- *    on the web site http://www.open-xchange.com/EN/legal/index.html.
- *
- *    Please make sure that third-party modules and libraries are used
- *    according to their respective licenses.
- *
- *    Any modifications to this package must retain all copyright notices
- *    of the original copyright holder(s) for the original code used.
- *
- *    After any such modifications, the original and derivative code shall remain
- *    under the copyright of the copyright holder(s) and/or original author(s)per
- *    the Attribution and Assignment Agreement that can be located at
- *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
- *    given Attribution for the derivative code and a license granting use.
- *
- *     Copyright (C) 2016-2020 OX Software GmbH
- *     Mail: info@open-xchange.com
- *
- *
- *     This program is free software; you can redistribute it and/or modify it
- *     under the terms of the GNU General Public License, Version 2 as published
- *     by the Free Software Foundation.
- *
- *     This program is distributed in the hope that it will be useful, but
- *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
- *     for more details.
- *
- *     You should have received a copy of the GNU General Public License along
- *     with this program; if not, write to the Free Software Foundation, Inc., 59
- *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Any use of the work other than as authorized under this license or copyright law is prohibited.
  *
  */
 
@@ -55,7 +27,6 @@ import java.io.IOException;
 import java.io.PrintWriter;
 import java.net.URI;
 import java.net.URISyntaxException;
-import java.util.Collection;
 import java.util.Date;
 import java.util.HashMap;
 import java.util.Map;
@@ -91,8 +62,8 @@ import com.openexchange.oidc.osgi.Services;
 import com.openexchange.session.Session;
 import com.openexchange.session.oauth.OAuthTokens;
 import com.openexchange.session.oauth.TokenRefreshConfig;
+import com.openexchange.sessiond.ExpirationReason;
 import com.openexchange.sessiond.SessionExceptionCodes;
-import com.openexchange.sessiond.SessionFilter;
 import com.openexchange.sessiond.SessiondService;
 import com.openexchange.tools.servlet.http.Cookies;
 import com.openexchange.tools.servlet.http.Tools;
@@ -124,8 +95,6 @@ public class OIDCTools {
 
     public static final String STATE = "state";
 
-    public static final String AUTOLOGIN_COOKIE_PREFIX = "open-xchange-oidc-";
-
     public static final String SESSION_COOKIE = "com.openexchange.oidc.SessionCookie";
 
     public static final String ACCESS_TOKEN = "access_token";
@@ -228,9 +197,16 @@ public class OIDCTools {
         SessionUtility.checkIP(session, request.getRemoteAddr());
         Map<String, Cookie> cookies = Cookies.cookieMapFor(request);
         Cookie secretCookie = cookies.get(LoginServlet.SECRET_PREFIX + session.getHash());
-        if (secretCookie == null || !session.getSecret().equals(secretCookie.getValue())) {
+        if (secretCookie == null) {
             LOG.debug("No secret cookie found for session: {}", session);
-            throw SessionExceptionCodes.SESSION_EXPIRED.create(session.getSessionID());
+            OXException oxe = SessionExceptionCodes.SESSION_EXPIRED.create(session.getSessionID());
+            oxe.setProperty(SessionExceptionCodes.OXEXCEPTION_PROPERTY_SESSION_EXPIRATION_REASON, ExpirationReason.NO_EXPECTED_SECRET_COOKIE.getIdentifier());
+            throw oxe;
+        } else if (!session.getSecret().equals(secretCookie.getValue())) {
+            LOG.debug("No secret cookie found for session: {}", session);
+            OXException oxe = SessionExceptionCodes.SESSION_EXPIRED.create(session.getSessionID());
+            oxe.setProperty(SessionExceptionCodes.OXEXCEPTION_PROPERTY_SESSION_EXPIRATION_REASON, ExpirationReason.SECRET_MISMATCH.getIdentifier());
+            throw oxe;
         }
     }
 
@@ -273,18 +249,18 @@ public class OIDCTools {
     }
 
     /**
-     * Load the OIDC auto-login cookie with the given {@link HttpServletRequest} informations.
+     * Load the session cookie with the given {@link HttpServletRequest} informations.
      *
      * @param request The {@link HttpServletRequest} with needed information
      * @param loginConfiguration The {@link LoginConfiguration} to get additional information from
-     * @return The auto-login cookie or null
+     * @return The session cookie or null
      * @throws OXException If something fails
      */
-    public static Cookie loadAutologinCookie(HttpServletRequest request, LoginConfiguration loginConfiguration) throws OXException {
-        LOG.trace("loadAutologinCookie(HttpServletRequest request: {}, LoginConfiguration loginConfiguration)", request.getRequestURI());
+    public static Cookie loadSessionCookie(HttpServletRequest request, LoginConfiguration loginConfiguration) throws OXException {
+        LOG.trace("loadSessionCookie(HttpServletRequest request: {}, LoginConfiguration loginConfiguration)", request.getRequestURI());
         String hash = calculateHash(request, loginConfiguration);
         Map<String, Cookie> cookies = Cookies.cookieMapFor(request);
-        return cookies.get(OIDCTools.AUTOLOGIN_COOKIE_PREFIX + hash);
+        return cookies.get(LoginServlet.SESSION_PREFIX + hash);
     }
 
     public static String calculateHash(HttpServletRequest request, LoginConfiguration loginConfiguration) throws OXException {
@@ -410,20 +386,14 @@ public class OIDCTools {
     /**
      * Load a session from the given {@link Cookie}.
      *
-     * @param oidcAtologinCookie The cookie where the session is stored.
+     * @param sessionCookie The cookie where the session is stored.
      * @param request Used to validate the found session.
      * @return The loaded session or null, if no session could be found or the validation failed.
-     * @throws OXException If an error occurs while filtering, when finding sessions
      */
-    public static Session getSessionFromAutologinCookie(Cookie oidcAtologinCookie, HttpServletRequest request) throws OXException {
-        LOG.trace("getSessionFromAutologinCookie(Cookie oidcAtologinCookie: {})", oidcAtologinCookie.getValue());
-        Session session = null;
+    public static Session getSessionFromSessionCookie(Cookie sessionCookie, HttpServletRequest request) {
+        LOG.trace("getSessionFromSessionCookie(Cookie sessionCookie: {})", sessionCookie.getValue());
         SessiondService sessiondService = Services.getService(SessiondService.class);
-        Collection<String> sessions = sessiondService.findSessions(SessionFilter.create("(" + OIDCTools.SESSION_COOKIE + "=" + oidcAtologinCookie.getValue() + ")"));
-        if (!sessions.isEmpty()) {
-            session = sessiondService.getSession(sessions.iterator().next());
-        }
-
+        Session session = sessiondService.getSession(sessionCookie.getValue());
         if (session == null) {
             return null;
         }
